package net.wanji.common.tool.dbtools;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import net.wanji.common.framework.exception.NodeException;
import org.apache.commons.lang.StringUtils;

import java.math.BigDecimal;
import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static net.wanji.common.framework.Constants.PostGreSqlColumn.*;
import static net.wanji.common.framework.Constants.PostGreSqlColumn.PATH;
import static net.wanji.common.framework.Constants.ServicePublishConstants.SqlConstants.*;

/**
 * 数据库相关处理帮助类
 *
 * @author guoliang.dong@wanji.com
 *
 */
public class DBUtils {

    /**
     * @param runtimeParams 运行时参数。对应占位符的实际值。
     * @param pluginJson    pluginJson。
     * @return 替换为真实值的pluginJson的map格式。
     */
    public static List<Map<String, Object>> tranAndGetPluginJsonMaps(Map<String, Object> runtimeParams,
                                                              JSONArray pluginJson) throws NodeException {
        // 处理数据库字段
        deleteNoParam(pluginJson, runtimeParams);

        List<Map<String, Object>> result = new ArrayList<>();
        try {
            if (!pluginJson.isEmpty()) {
                for (int i = 0; i < pluginJson.size(); i++) {
                    JSONObject map = pluginJson.getJSONObject(i);
                    String variable = map.getString(FIELD_VALUE);
                    Object value = runtimeParams.get(variable);
                    map.put(FIELD_VALUE, value);
                    result.add(map.toJavaObject(new TypeReference<Map<String, Object>>() {
                    }));
                }
            }
            return result;
        } catch (Exception e) {
            throw new NodeException("处理数据库字段异常");
        }
    }

    // 判断value是否为null。
    private static boolean checkNull(Object value) {
        return value == null || StringUtils.isBlank(String.valueOf(value));
    }

    /**
     * @return void
     * @throws
     * @Author wanji
     * @Description 删除无参字段
     * @Date 11:53 2020/6/9
     * @Param [typesAndValues, condColumns, operations]
     */
    private static void deleteNoParam(JSONArray pluginJson, Map<String, Object> runtimeParams) throws NodeException {
        for (int i = 0; i < pluginJson.size(); i++) {
            JSONObject map = pluginJson.getJSONObject(i);
            if (map == null) {
                pluginJson.remove(i);
                if (pluginJson.size() > 0) {
                    deleteNoParam(pluginJson, runtimeParams);
                }
                return;
            } else {
                String value = String.valueOf(map.get(FIELD_VALUE));
                boolean flag = false;
                if (map.get(FIELD_FLAG) != null) {
                    try {
                        flag = (boolean) map.get(FIELD_FLAG);
                    } catch (Exception e) {
                        throw new NodeException("flag格式转换错误");
                    }
                }
                if (StringUtils.isBlank(value) ||
                        runtimeParams == null ||
                        checkNull(runtimeParams.get(value))) {
                    if (flag) {
                        throw new NodeException("请填写必要的条件: " + value);
                    } else {
                        pluginJson.remove(i);
                        if (pluginJson.size() > 0) {
                            deleteNoParam(pluginJson, runtimeParams);
                        }
                        return;
                    }
                }
            }
        }
    }

    private static Object typeHandler(String fieldType, String fieldValue) throws NodeException{
        switch (fieldType) {
            case FLOAT4:
                return Float.valueOf(fieldValue);
            case FLOAT8:
            case MONEY:
                return Double.valueOf(fieldValue);
            case INT4:
            case INT2:
                return Integer.valueOf(fieldValue);
            case BOOL:
            case net.wanji.common.framework.Constants.PostGreSqlColumn.BIT:
                return Boolean.valueOf(fieldValue);
            case NUMERIC:
                return new BigDecimal(fieldValue);
            case INT8:
                return Long.valueOf(fieldValue);
            case TIMESTAMP:
                return Timestamp.valueOf(fieldValue);
            case TIME:
                return Time.valueOf(fieldValue);
            case DATE:
                return Date.valueOf(fieldValue);
            case BYTEA:
            case TEXT:
            case BPCHAR:
            case net.wanji.common.framework.Constants.PostGreSqlColumn.VARCHAR:
            case BOX:
            case CIDR:
            case INET:
            case MACADDR:
            case VARBIT:
            case CIRCLE:
            case INTERVAL:
            case LINE:
            case LSEG:
            case POLYGON:
            case POINT:
            case PATH:
                return fieldValue;
            default:
                throw new NodeException("[PostgresqlTypeHandler] 未支持的类型:" + fieldType);
        }
    }

    public static Object[] typeHandler(List<Map<String, Object>> paramList) throws NodeException {
        List<Object> list = new ArrayList<>();
        for (Map<String, Object> paramMap : paramList) {
            String fieldType = String.valueOf(paramMap.get(FIELD_TYPE));
            String fieldValue = String.valueOf(paramMap.get(FIELD_VALUE));

            if (net.wanji.common.utils.tool.StringUtils.equalsIgnoreCase(fieldValue, "null")) {
                list.add(null);
            } else {
                    list.add(typeHandler(fieldType, fieldValue));
            }
        }
        return list.toArray();
    }
}
