package net.wanji.common.tool.dbtools;

import java.util.List;
import java.util.Map;

import static net.wanji.common.framework.Constants.ServicePublishConstants.SqlConstants.*;

/**
 * Oracle sql 构造器。
 *
 * @Author wanji
 * @Date 2020/3/13 11:48
 */
public class Oracle {
    private String select;
    private String delete;
    private String update;
    private String set;
    private String columns;
    private String from;
    private String where;
    private String order;

    private Oracle(SqlBuilder sqlBuilder) {
        this.select = sqlBuilder.select;
        this.delete = sqlBuilder.delete;
        this.update = sqlBuilder.update;
        this.set = sqlBuilder.set;
        this.columns = sqlBuilder.columns;
        this.from = sqlBuilder.from;
        this.where = sqlBuilder.where;
        this.order = sqlBuilder.order;
    }

    public static class SqlBuilder {
        private String select;
        private String delete;
        private String update;
        private String set;
        private String columns;
        private String from;
        private String where;
        private String order;

        public SqlBuilder select() {
            this.select = "SELECT ";
            return this;
        }

        public SqlBuilder delete() {
            this.delete = "DELETE ";
            return this;
        }

        public SqlBuilder update(String tableName) {
            this.update = "UPDATE " + "`" + tableName + "`";
            return this;
        }

        // 构造update中的set片段
        public SqlBuilder buildSet(List<Map<String, Object>> updateParams) {
            StringBuilder setBuilder = new StringBuilder(" SET ");
            for (int i = 0; i < updateParams.size(); i++) {
                if (updateParams.size() == (i + 1)) {
                    setBuilder.append("\"").append(updateParams.get(i).get(FIELD_NAME_EN).toString()).append("\"").append("=? ");
                } else {
                    setBuilder.append("\"").append(updateParams.get(i).get(FIELD_NAME_EN).toString()).append("\"").append("=?, ");
                }
            }
            this.set = setBuilder.toString();
            return this;
        }

        public SqlBuilder buildAllColumns() {
            this.columns = "* ";
            return this;
        }

        public SqlBuilder buildCount() {
            this.columns = "count(*) ";
            return this;
        }

        public SqlBuilder buildFrom(String table) {
            this.from = "FROM " + "`" + table + "`";
            return this;
        }

        // 根据不同的操作符来拼接where片段
        public SqlBuilder buildWhere(List<Map<String, Object>> whereParams) {
            StringBuilder whereBuilder = new StringBuilder(" WHERE 1=1 ");
            for (Map<String, Object> map : whereParams) {
                String operation = map.get(OPERATION).toString().toUpperCase();
                String fieldValue = map.get(FIELD_VALUE).toString();
                String conditionCol = map.get(FIELD_NAME_EN).toString();
                if (BETWEEN_OPERATION.equalsIgnoreCase(operation)) {
                    whereBuilder.append("AND ").append("\"").append(conditionCol).append("\"").append(" ")
                            .append(operation).append(" ? AND ").append("? ");
                } else if (IN_OPERATION.equalsIgnoreCase(operation)) {
                    whereBuilder.append("AND ").append("\"").append(conditionCol).append("\"").append(" ")
                            .append(operation);
                    String[] split = fieldValue.split(",");
                    int length = split.length;
                    for (int j = 0; j < length; j++) {
                        if (j == 0) {
                            whereBuilder.append(" (");
                        } else {
                            whereBuilder.append(",");
                        }
                        whereBuilder.append("?");
                        if (j == length - 1) {
                            whereBuilder.append(") ");
                        }
                    }
                } else {
                    whereBuilder.append("AND ").append("\"").append(conditionCol).append("\"").append(" ")
                            .append(operation).append(" ? ");
                }
            }

            this.where = whereBuilder.toString();
            return this;
        }

        // 根据不同的操作符来拼接 order 片段
        public SqlBuilder buildOrder(List<Map<String, String>> orderParams) {
            StringBuilder orderBuilder = new StringBuilder();
            if (orderParams != null && !orderParams.isEmpty()) {
                orderBuilder.append(" ORDER BY ");
                for (Map<String, String> orderParam : orderParams) {
                    String operation = orderParam.get(OPERATION);
                    String fieldNameEn = orderParam.get(FIELD_NAME_EN);
                    if (DESC_OPERATION.equalsIgnoreCase(operation)) { // 降序
                        orderBuilder.append(fieldNameEn).append(" ").append(DESC_OPERATION).append(" ,");
                    } else { // 默认 升序排序
                        orderBuilder.append(fieldNameEn).append(" ").append(ASC_OPERATION).append(" ,");
                    }
                }
                orderBuilder = new StringBuilder(orderBuilder.toString().substring(0, orderBuilder.toString().length() - 1));
            }

            this.order = orderBuilder.toString();
            return this;
        }

        public Oracle build() {
            return new Oracle(this);
        }
    }

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder();
        if (select != null) {
            builder.append(select);
        }
        if (delete != null) {
            builder.append(delete);
        }
        if (update != null) {
            builder.append(update);
        }
        if (set != null) {
            builder.append(set);
        }
        if (columns != null) {
            builder.append(columns);
        }
        if (from != null) {
            builder.append(from);
        }
        if (where != null) {
            builder.append(where);
        }
        if (order != null) {
            builder.append(order);
        }
        return builder.toString();
    }
}
