package net.wanji.common.tool.drools;

import net.wanji.common.utils.tool.MD5Util;
import com.google.common.collect.Maps;
import com.googlecode.aviator.AviatorEvaluator;
import org.apache.commons.collections4.MapUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.Map;
import java.util.Objects;

/**
 * <>
 *
 * @author wanji
 * 2020/6/13 9:01
 */
public class AviatorUtils {

    private static final Logger log = LogManager.getLogger(AviatorUtils.class);

    private static final ThreadLocal<Map<String, Class<?>>> THREAD_LOCAL_FUNCTION = new ThreadLocal<>();

    private static final ThreadLocal<String> THREAD_LOCAL_AVIATOR_MAP = new ThreadLocal<>();

    public static boolean test() {
        return true;
    }

    public static boolean assertor(String exp, Object... args) {
        Map<String, Object> map = AviatorEvaluator.newEnv(args);
        Object object;
        if (Objects.isNull(map)) {
            object = AviatorEvaluator.getInstance().compile(Objects.requireNonNull(MD5Util.md5(exp)), exp, true);
        } else {
            object = AviatorEvaluator.getInstance().compile(Objects.requireNonNull(MD5Util.md5(exp)), exp, true).execute(map);
        }
        if (Objects.isNull(object)) {
            return true;
        }
        Class<?> clazz = object.getClass();
        if (clazz.isAssignableFrom(Boolean.class) || clazz.isAssignableFrom(boolean.class)) {
            return (Boolean) object;
        } else {
            return false;
        }
    }

    public static Object execute(String exp, Object... args) {
        Map<String, Object> map = AviatorEvaluator.newEnv(args);
        return AviatorEvaluator.getInstance().compile(Objects.requireNonNull(MD5Util.md5(exp)), exp, true).execute(map);
    }

    public static String genNamespace(String actionId, String beanName) {
        String action = Objects.isNull(actionId) ? "Drools" : actionId;
        return beanName + action;
    }

    public static String addStaticFunctions(String actionId, String beanName, String path) {
        try {
            Class<?> clazz = Class.forName(path);
            return addStaticFunctions(actionId, beanName, clazz);
        } catch (Exception e) {
            log.error("AviatorEvaluator addStaticFunctions error!", e);
            return "";
        }
    }

    public static void addAviatorMap(String aviatorMap) {
        THREAD_LOCAL_AVIATOR_MAP.set(aviatorMap);
    }

    public static String getAviatorMap() {
        return THREAD_LOCAL_AVIATOR_MAP.get();
    }

    public static void removeAviatorMap() {
        THREAD_LOCAL_AVIATOR_MAP.remove();
    }

    public static String addStaticFunctions(String actionId, String beanName, Class<?> clazz) {
        String namespace = genNamespace(actionId, beanName);
        Map<String, Class<?>> map = THREAD_LOCAL_FUNCTION.get();
        if (MapUtils.isEmpty(map)) {
            map = Maps.newHashMap();
        }
        map.put(namespace, clazz);
        THREAD_LOCAL_FUNCTION.set(map);
        return namespace;
    }

    public static void registerStaticFunctions() {
        try {
            Map<String, Class<?>> map = THREAD_LOCAL_FUNCTION.get();
            if (MapUtils.isEmpty(map)) {
                return;
            }
            for (Map.Entry<String, Class<?>> entry : map.entrySet()) {
                try {
                    AviatorEvaluator.getInstance().addStaticFunctions(entry.getKey(), entry.getValue());
                } catch (Exception e) {
                    log.error("AviatorEvaluator addStaticFunctions error!", e);
                }
            }
        } catch (Exception e) {
            log.error("AviatorEvaluator addStaticFunctions error!", e);
            throw new RuntimeException(e);
        } finally {
            THREAD_LOCAL_FUNCTION.remove();
        }
    }
}
