package net.wanji.common.tool.estool;

import net.wanji.common.framework.spring.ServiceBeanContext;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.elasticsearch.client.transport.TransportClient;
import org.springframework.boot.autoconfigure.AutoConfigureAfter;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.util.HashMap;
import java.util.Map;

/**
 * @Description: es client bean管理
 * @Author wanji
 * @Date 2019/12/10 10:01
 */
@Configuration
@AutoConfigureAfter
@Slf4j
public class EsConfiguration {

    private static TransportClient transportClient = null;

    public static TransportClient getTransportClient() {
        if (transportClient == null) {
            transportClient = getEsClient();
        }
        return transportClient;
    }

    /**
     * @Description: es获取配置文件中的name和url
     * @Author wanji
     * @Date 2019/12/13 11:02
     */
    private static TransportClient getEsClient() {
        EsProperties esProperties = ServiceBeanContext.getBean(EsProperties.class);
        if (StringUtils.isEmpty(esProperties.getUrl()) || StringUtils.isEmpty(esProperties.getName())) {
            log.error("es consumer start error: esProperties config error");
            return null;
        }
        Map address = new HashMap();
        //按照逗号切分不同的url
        String[] esUrlArray = StringUtils.split(esProperties.getUrl(), ",");
        if (null == esUrlArray || esUrlArray.length == 0) {
            log.error("es consumer start error: esProperties config error");
            return null;
        }
        for (String esUrl : esUrlArray) {
            if (esUrl.contains(":")) {
                //按照冒号切分不同的url
                String[] esUrlItem = StringUtils.split(esUrl, "\\:");
                address.put(String.valueOf(esUrlItem[0]), Integer.valueOf(esUrlItem[1]));
            }
        }
        return EsUtil.getInstance().esClient(esProperties.getName(), address);
    }

    @Bean(value = "esUtil")
    public EsUtil getEsUtil() {
        return EsUtil.getInstance();
    }

}
