package net.wanji.common.tool.http;

import lombok.extern.slf4j.Slf4j;
import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.client.methods.*;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicHeader;
import org.apache.http.ssl.SSLContexts;
import org.apache.http.util.EntityUtils;

import javax.net.ssl.SSLContext;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.KeyStore;

/**
 *
 * HTTPS 双向认证 - direct into cacerts
 * Created by wanji on 2019/7/10.
 */
@Slf4j
public class HttpsClient {

    private static String KEY_TOOL = "${JAVA_HOME}/jre/bin/keytool";
    private static String KEY_STORE = "\"${JAVA_HOME}/jre/lib/security/cacerts\"";

    public static void importCrt(String path, String keyPass, String aliasName) {
        //"/bin/sh","-c"   / "cmd", "c"
        String[] shell = {"/bin/sh","/-c",KEY_TOOL+" -import -keystore "+KEY_STORE+" -storepass changeit -keypass "+keyPass+" -alias "+aliasName+" -file "+path+" -noprompt"};
        try {
            Runtime.getRuntime().exec(shell);
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /**
     * HTTPS协议GET调用
     *
     * @param url
     * @param clientCertPath
     * @param secretKey
     * @param headers
     * @return
     * @throws Exception
     */
    public static String sslRequestGet(String url, String clientCertPath, String secretKey, Header[] headers) throws Exception {
        CloseableHttpClient httpclient = getHttpClient(clientCertPath, secretKey);
        try {
            HttpGet httpget = new HttpGet(url);
            // 设置heander等
            headers = judgeHeaders(headers);
            httpget.setHeaders(headers);
            CloseableHttpResponse response = httpclient.execute(httpget);
            try {
                HttpEntity entity = response.getEntity();
                // 返回结果
                String jsonStr = EntityUtils.toString(response.getEntity(), "UTF-8");
                EntityUtils.consume(entity);
                return jsonStr;
            } finally {
                response.close();
            }
        } finally {
            httpclient.close();
        }
    }

    /**
     * HTTPS协议DELETE调用
     *
     * @param url
     * @param clientCertPath
     * @param secretKey
     * @param headers
     * @return
     * @throws Exception
     */
    public static String sslRequestDelete(String url, String clientCertPath, String secretKey, Header[] headers) throws Exception {
        CloseableHttpClient httpclient = getHttpClient(clientCertPath, secretKey);
        try {
            HttpDelete httpDelete = new HttpDelete(url);
            // 设置heander等
            headers = judgeHeaders(headers);
            httpDelete.setHeaders(headers);
            CloseableHttpResponse response = httpclient.execute(httpDelete);
            try {
                HttpEntity entity = response.getEntity();
                // 返回结果
                String jsonStr = EntityUtils.toString(response.getEntity(), "UTF-8");
                EntityUtils.consume(entity);
                return jsonStr;
            } finally {
                response.close();
            }
        } finally {
            httpclient.close();
        }
    }

    private static CloseableHttpClient getHttpClient(String clientCertPath, String secretKey) throws Exception {
        if (clientCertPath == null || "null".equals(clientCertPath) || "".equals(clientCertPath)) {
            return HttpClients.custom().build();
        }
        KeyStore keyStore = KeyStore.getInstance("PKCS12");
        InputStream instream = new FileInputStream(new File(clientCertPath));
        try {
            // 这里就指的是KeyStore库的密码
            keyStore.load(instream, secretKey.toCharArray());
        } finally {
            instream.close();
        }
        SSLContext sslcontext = SSLContexts.custom()
                .loadKeyMaterial(keyStore, secretKey.toCharArray())
                .build();
        SSLConnectionSocketFactory sslsf = new SSLConnectionSocketFactory(sslcontext
                // supportedProtocols ,这里可以按需要设置
                , new String[] { "TLSv1" }
                , null
                , SSLConnectionSocketFactory.getDefaultHostnameVerifier());
        return HttpClients.custom().setSSLSocketFactory(sslsf).build();

    }

    /**
     * HTTPS协议POST调用
     *
     * @param url
     * @param clientCertPath
     * @param secretKey
     * @param headers
     * @param bodyStr
     * @return
     * @throws Exception
     */
    public static String sslRequestPost(String url, String clientCertPath, String secretKey, Header[] headers, String bodyStr) throws Exception {
        CloseableHttpClient httpclient = getHttpClient(clientCertPath, secretKey);
        try {
            HttpPost httpPost = new HttpPost(url);
            // 设置一些heander等
            headers = judgeHeaders(headers);
            httpPost.setHeaders(headers);
            httpPost.setEntity(new StringEntity(bodyStr, "UTF-8"));
            CloseableHttpResponse response = httpclient.execute(httpPost);
            try {
                HttpEntity entity = response.getEntity();
                // 返回结果
                String jsonStr = EntityUtils.toString(response.getEntity(), "UTF-8");
                EntityUtils.consume(entity);
                return jsonStr;
            } finally {
                response.close();
            }
        } finally {
            httpclient.close();
        }
    }

    /**
     * HTTPS协议PUT调用
     *
     * @param url
     * @param clientCertPath
     * @param secretKey
     * @param headers
     * @param bodyStr
     * @return
     * @throws Exception
     */
    public static String sslRequestPut(String url, String clientCertPath, String secretKey, Header[] headers, String bodyStr) throws Exception {
        CloseableHttpClient httpclient = getHttpClient(clientCertPath, secretKey);
        try {
            HttpPut httpPut = new HttpPut(url);
            // 设置一些heander等
            headers = judgeHeaders(headers);
            httpPut.setHeaders(headers);
            httpPut.setEntity(new StringEntity(bodyStr, "UTF-8"));
            CloseableHttpResponse response = httpclient.execute(httpPut);
            try {
                HttpEntity entity = response.getEntity();
                // 返回结果
                String jsonStr = EntityUtils.toString(response.getEntity(), "UTF-8");
                EntityUtils.consume(entity);
                return jsonStr;
            } finally {
                response.close();
            }
        } finally {
            httpclient.close();
        }
    }

    /**
     * 如果header为空，初始化一些必要的参数
     *
     * @param headers
     */
    private static Header[] judgeHeaders(Header[] headers) {
        if (headers == null || headers.length == 0) {
            headers = new Header[1];
            headers[0] = new BasicHeader("Content-Type", "application/json;chartset=UTF-8");
        }
        return headers;
    }
}
