package net.wanji.common.tool.jobtool;

import lombok.extern.slf4j.Slf4j;

import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;

/**
 * @ClassName RetryTemplate
 * @Description
 * @Author wanji
 * @Date 2020/7/13 10:00
 * @Version 1.0
 */
@Slf4j
public abstract class RetryTemplate {

    private static final int DEFAULT_RETRY_TIME = 1;

    private int retryTime = DEFAULT_RETRY_TIME;

    // 重试的睡眠时间
    private int sleepTime = 0;

    public int getSleepTime() {
        return sleepTime;
    }

    public RetryTemplate setSleepTime(int sleepTime) {
        if (sleepTime < 0) {
            throw new IllegalArgumentException("sleepTime should equal or bigger than 0");
        }

        this.sleepTime = sleepTime;
        return this;
    }

    public int getRetryTime() {
        return retryTime;
    }

    public RetryTemplate setRetryTime(int retryTime) {
        if (retryTime <= 0) {
            throw new IllegalArgumentException("retryTime should bigger than 0");
        }
        this.retryTime = retryTime;
        return this;
    }

    /**
     * 重试的业务执行代码
     * 失败时请抛出一个异常
     */
    protected abstract Object doBiz() throws Exception;

    public Object execute() throws InterruptedException {
        for (int i = 0; i < retryTime; i++) {
            try {
                return doBiz();
            } catch (Exception e) {
                log.error("业务执行出现异常，e: {}", e);
                Thread.sleep(sleepTime);
            }
        }
        return null;
    }

    public Object submit(ExecutorService executorService) {
        if (executorService == null) {
            throw new IllegalArgumentException("please choose executorService!");
        }
        return executorService.submit((Callable) () -> execute());
    }

    public static void main(String[] args) {
        Object ans = null;
        try {
            ans = new RetryTemplate() {
                @Override
                protected Object doBiz() throws Exception {
                    int temp = (int) (Math.random() * 10);
                    System.out.println(temp);

                    if (temp > 3) {
                        throw new Exception("generate value bigger then 3! need retry");
                    }

                    return temp;
                }
            }.setRetryTime(10).setSleepTime(10000).execute();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        System.out.println(ans);
    }
}
