package net.wanji.common.tool.nginxtool;

import net.wanji.common.framework.exception.DubboProviderException;
import lombok.extern.slf4j.Slf4j;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Enumeration;
import java.util.Properties;

/**
 * @Author wanji
 * @Date 2020/2/11 15:47
 **/
@Slf4j
public class OperationMonitorUtil {

    /**
     * @param ipAddress
     * @return boolean
     * @Description: 测试指定的ip是否能ping通
     * @Author wanji
     * @Date 2020/2/12 9:23
     **/
    public static boolean ping(String ipAddress) throws Exception {
        int timeOut = 3000;
        boolean status = InetAddress.getByName(ipAddress).isReachable(timeOut);
        return status;
    }


    /**
     * @param processName
     * @return boolean
     * @Description:
     * @Author wanji
     * @Date 2020/2/11 18:28
     **/
    static boolean findProcess(String processName) {
        BufferedReader bufferedReader = null;
        try {
            Properties prop = System.getProperties();
            Process proc = null;
            // 获取操作系统名称
            String os = prop.getProperty("os.name");
            if (os != null && os.toLowerCase().startsWith("lin")) {
                proc = Runtime.getRuntime().exec("ps -ef");
            } else {
                proc = Runtime.getRuntime().exec("tasklist");
            }
            bufferedReader = new BufferedReader(new InputStreamReader(proc.getInputStream()));
            String line = null;
            while ((line = bufferedReader.readLine()) != null) {
                if (line.contains(processName)) {
                    return true;
                }
            }
            return false;
        } catch (Exception ex) {
            log.error("get procedure whetherRunning failed!");
            return false;
        } finally {
            if (bufferedReader != null) {
                try {
                    bufferedReader.close();
                } catch (Exception ex) {
                    log.error("bufferedReader close failed!");
                }
            }
        }
    }

    /**
     * @param
     * @return java.lang.String
     * @Description:
     * @Author wanji
     * @Date 2020/2/11 18:29
     **/
    public static String getSystemLocalIp() throws Exception {
        String ip = null;
        String osname = System.getProperties().getProperty("os.name");
        try {
            // 针对window系统
            if (osname.startsWith("Windows")) {
                ip = InetAddress.getLocalHost().getHostAddress();
                // 针对linux系统
            } else {
                ip = getUnixLocalIp();
            }
            if (null == ip) {
                throw new DubboProviderException("主机的ip地址未知");
            }
        } catch (SocketException e) {
            log.error("获取本机ip错误" + e.getMessage());
            throw new DubboProviderException("获取本机ip错误" + e.getMessage());
        }
        return ip;
    }

    /**
     * @param
     * @return java.net.InetAddress
     * @Description: 获取Linux系统下本机ip地址
     * @Author wanji
     * @Date 2020/2/11 18:08
     **/
    private static String getUnixLocalIp() throws SocketException {
        String ip = "";
        try {
            for (Enumeration<NetworkInterface> en = NetworkInterface.getNetworkInterfaces(); en.hasMoreElements(); ) {
                NetworkInterface intf = en.nextElement();
                String name = intf.getName();
                if (!name.contains("docker") && !name.contains("lo")) {
                    for (Enumeration<InetAddress> enumIpAddr = intf.getInetAddresses(); enumIpAddr.hasMoreElements(); ) {
                        InetAddress inetAddress = enumIpAddr.nextElement();
                        if (!inetAddress.isLoopbackAddress()) {
                            String ipaddress = inetAddress.getHostAddress().toString();
                            if (!ipaddress.contains("::") && !ipaddress.contains("0:0:") && !ipaddress.contains("fe80")) {
                                ip = ipaddress;
                                log.info(ipaddress);
                            }
                        }
                    }
                }
            }
        } catch (SocketException ex) {
            log.error("获取IP失败");
            ip = "127.0.0.1";
            ex.printStackTrace();
        }
        return ip;
    }

}
